﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace Parallel
{
	internal sealed class CountDown : IDisposable
	{
		private int count;
		private readonly ManualResetEvent manualEvent;

		public CountDown(int count) 
		{
			this.count = count;
			manualEvent = new ManualResetEvent(false);
		}

		public void Signal()
		{
			if (Interlocked.Decrement(ref count) == 0)
				manualEvent.Set();
		}

		public void Wait() 
		{
			manualEvent.WaitOne();
		}


		public void Dispose()
		{
			Dispose(true);
			GC.SuppressFinalize(this);
		}

		private void Dispose(bool disposing)
		{
			manualEvent.Close();
		}

		~CountDown()
		{
			Dispose(false);
		}


	}


}
